@@ -36,19 +36,28 @@ class UrlTest(TestCase):
         return getattr(self.client, method)(**kwargs)
     
     def verify_response(self, response, templates=[],
-                        view_name=None, status_code=200):
+                        view_name=None, contain_list=[],
+                        not_contain_list=[], status_code=200):
         self.assertEqual(response.status_code, status_code)
         for template_name in templates:
             self.assertTemplateUsed(response, template_name, 
                     'Template Not Used: %s' % template_name)
         self.verify_view(response, view_name)
- 
+        for contain in contain_list:
+            self.assertContains(response, **contain)
+        for not_contain in not_contain_list:
+            self.assertNotContains(response, **not_contain)
+            
     def request_response(self, path, data={}, ajax=False, 
                          templates=[], view_name=None,
+                         contain_list=[], not_contain_list=[],
                          status_code=200):
         response = self.request(path, data, ajax)
-        self.verify_response(response, templates, 
-                             view_name, status_code)
+        self.verify_response(response=response, templates=templates, 
+                             view_name=view_name, contain_list=contain_list,
+                             not_contain_list=not_contain_list,
+                             status_code=status_code)
+        return response
 
     def verify_view(self, response, view_name):
         if view_name:
@@ -64,6 +73,7 @@ class UrlTest(TestCase):
         self.verify_view(response, view_name)
         self.assertRedirects(response, expected_url, status_code,
                              target_status_code)
+        return response
 
 class AdminTest(UrlTest):
     
@@ -81,7 +91,7 @@ class AdminTest(UrlTest):
                      'admin/base.html']
         path = '%s/%s/' % (self.base_url, model)
         view_name = 'changelist_view'
-        self.request_response(path, templates=templates, view_name=view_name)
+        return self.request_response(path, templates=templates, view_name=view_name)
         
     def change_form_test(self, model, instance_id):
         templates = ['admin/change_form.html', 
@@ -90,7 +100,7 @@ class AdminTest(UrlTest):
                      'admin/includes/fieldset.html']
         path = '%s/%s/%s/' % (self.base_url, model, instance_id)
         view_name = 'change_view'
-        self.request_response(path, templates=templates, view_name=view_name)
+        return self.request_response(path, templates=templates, view_name=view_name)
 
 class ViewTest(FactoryTest):
     
@@ -127,12 +137,14 @@ class ViewTest(FactoryTest):
                                 credentials, arguments)
         self.verify_response(response, contain_list,
                              not_contain_list, status_code)
+        return response
     
 class AjaxTest(ViewTest):
         
     def request(self, path, view, data={},
                 credentials = None, arguments = {}):
-        return super(AjaxTest, self).request(path, view[0], data,
+        view = view[0] if hasattr(view,'__iter__') else view
+        return super(AjaxTest, self).request(path, view, data,
                     True, credentials, arguments)
     
     def verify_dajax_response(self, json_response_string, dajax_data):
@@ -147,3 +159,16 @@ class AjaxTest(ViewTest):
         response = self.request(path, view, data,
                                 credentials, arguments)
         self.verify_dajax_response(response, dajax_data)
+        return response
+
+class ModelsTest(TestCase):
+    
+    def verify_natural_keys(self, models):
+        for Model in models:
+            obj = Model.objects.all()[0]
+            self.verify_natural_key_for_object(obj, Model)
+    
+    def verify_natural_key_for_object(self, obj, Model):
+        nk = obj.natural_key()
+        self.assertEqual(Model.objects.get_by_natural_key(*nk), obj)
+
